'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var dateFns = require('date-fns');
var react = require('react');

var CalendarViewtype;

(function (CalendarViewtype) {
  CalendarViewtype["Month"] = "month";
  CalendarViewtype["Week"] = "week";
  CalendarViewtype["Day"] = "day";
})(CalendarViewtype || (CalendarViewtype = {}));

var CalendarViewType = CalendarViewtype;

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var defineProperty = _defineProperty;

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

var arrayWithHoles = _arrayWithHoles;

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

var iterableToArrayLimit = _iterableToArrayLimit;

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

var arrayLikeToArray = _arrayLikeToArray;

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return arrayLikeToArray(o, minLen);
}

var unsupportedIterableToArray = _unsupportedIterableToArray;

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var nonIterableRest = _nonIterableRest;

function _slicedToArray(arr, i) {
  return arrayWithHoles(arr) || iterableToArrayLimit(arr, i) || unsupportedIterableToArray(arr, i) || nonIterableRest();
}

var slicedToArray = _slicedToArray;

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return arrayLikeToArray(arr);
}

var arrayWithoutHoles = _arrayWithoutHoles;

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter);
}

var iterableToArray = _iterableToArray;

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var nonIterableSpread = _nonIterableSpread;

function _toConsumableArray(arr) {
  return arrayWithoutHoles(arr) || iterableToArray(arr) || unsupportedIterableToArray(arr) || nonIterableSpread();
}

var toConsumableArray = _toConsumableArray;

function arrayOf(count) {
  return toConsumableArray(new Array(count).keys());
}

var id = 0;
function generateID(prefix) {
  id += 1;
  return "".concat(prefix, "-").concat(id);
}

function attachKeyToArray(arr, keyPrefix) {
  return arr.map(function (value) {
    return {
      key: generateID(keyPrefix),
      value: value
    };
  });
}

function parseDate(date) {
  return {
    year: date.getFullYear(),
    month: date.getMonth(),
    day: date.getDate()
  };
}

function resetTimeOfDate(date) {
  return dateFns.setHours(dateFns.setMinutes(dateFns.setSeconds(dateFns.setMilliseconds(date, 0), 0), 0), 0);
}

function isSameDate(baseDate, targetDate) {
  var base = resetTimeOfDate(baseDate);
  var target = resetTimeOfDate(targetDate);
  return dateFns.isEqual(base, target);
}

function createCalendarInfo(date, weekStartsOn) {
  var _parseDate = parseDate(date),
      year = _parseDate.year,
      month = _parseDate.month,
      day = _parseDate.day;

  var startWeekdayInMonth = getStartWeekdayInMonth(date, weekStartsOn);
  var weeksInMonth = getWeeksInMonth(date, weekStartsOn);
  var weekendDays = arrayOf(7).map(function (index) {
    return dateFns.setDay(date, (index + weekStartsOn) % 7);
  });

  var getTargetDate = function getTargetDate(weekIndex, dayIndex) {
    var day = weekIndex * 7 + dayIndex - startWeekdayInMonth + 1;
    return new Date(year, month, day);
  };

  var getCurrentWeek = function getCurrentWeek() {
    var control = (day - 1) % 7 > startWeekdayInMonth ? 0 : -1;
    return Math.ceil(day / 7) + control;
  };

  var getMonth = function getMonth() {
    return arrayOf(weeksInMonth).map(function (weekIndex) {
      return getWeek(weekIndex);
    });
  };

  var getWeek = function getWeek(weekIndex) {
    return arrayOf(7).map(function (dayIndex) {
      var targetDate = getTargetDate(weekIndex, dayIndex);
      return {
        date: targetDate,
        isCurrentMonth: isCurrentMonth(targetDate),
        isCurrentDate: isCurrentDate(targetDate)
      };
    });
  };

  var isCurrentMonth = function isCurrentMonth(targetDate) {
    return dateFns.isSameMonth(targetDate, date) && dateFns.isSameYear(targetDate, date);
  };

  var isCurrentDate = function isCurrentDate(targetDate) {
    return isCurrentMonth(targetDate) && isSameDate(targetDate, date);
  };

  return {
    date: date,
    year: year,
    month: month,
    day: day,
    weekStartsOn: weekStartsOn,
    startWeekdayInMonth: startWeekdayInMonth,
    weeksInMonth: weeksInMonth,
    weekendDays: weekendDays,
    getTargetDate: getTargetDate,
    getCurrentWeek: getCurrentWeek,
    getWeek: getWeek,
    getMonth: getMonth,
    isCurrentMonth: isCurrentMonth,
    isCurrentDate: isCurrentDate
  };
}

function getStartWeekdayInMonth(date, weekStartsOn) {
  var monthStartsAt = (dateFns.startOfMonth(date).getDay() - weekStartsOn) % 7;
  return monthStartsAt < 0 ? monthStartsAt + 7 : monthStartsAt;
}

function getWeeksInMonth(date, weekStartsOn) {
  var _parseDate2 = parseDate(date),
      month = _parseDate2.month;

  var totalDaysOfMonth = dateFns.getDaysInMonth(month);
  return Math.ceil((weekStartsOn + totalDaysOfMonth) / 7);
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }
function useCalendar() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var defaultDate = options.defaultDate,
      _options$defaultWeekS = options.defaultWeekStart,
      defaultWeekStart = _options$defaultWeekS === void 0 ? 0 : _options$defaultWeekS,
      _options$defaultViewT = options.defaultViewType,
      defaultViewType = _options$defaultViewT === void 0 ? CalendarViewType.Month : _options$defaultViewT;
  var baseDate = react.useMemo(function () {
    return defaultDate ? new Date(defaultDate) : new Date();
  }, [defaultDate]);

  var _useState = react.useState(defaultWeekStart),
      _useState2 = slicedToArray(_useState, 2),
      weekStartsOn = _useState2[0],
      setWeekStartsOn = _useState2[1];

  var _useState3 = react.useState(baseDate),
      _useState4 = slicedToArray(_useState3, 2),
      currentDate = _useState4[0],
      setCurrentDate = _useState4[1];

  var _useState5 = react.useState(defaultViewType),
      _useState6 = slicedToArray(_useState5, 2),
      viewType = _useState6[0],
      setViewType = _useState6[1];

  var calendar = createCalendarInfo(currentDate, weekStartsOn);
  var weekendDays = calendar.weekendDays,
      getCurrentWeek = calendar.getCurrentWeek,
      getWeek = calendar.getWeek,
      getMonth = calendar.getMonth;
  var getHeaders = react.useCallback(function (viewType) {
    switch (viewType) {
      case 'month':
      case 'week':
        return {
          weekDays: attachKeyToArray(weekendDays, 'weekdays')
        };

      case 'day':
      default:
        return {
          weekDays: attachKeyToArray([currentDate], 'weekdays')
        };
    }
  }, [currentDate, weekendDays]);
  var getBody = react.useCallback(function (viewType) {
    var selectedDate = {
      date: currentDate,
      isCurrentDate: false,
      isCurrentMonth: true
    };
    var currentWeek = getCurrentWeek();
    var monthWeeks = attachKeyToArray(getMonth().map(function (week) {
      return attachKeyToArray(week, 'days');
    }), 'weeks');
    var weekWeeks = attachKeyToArray([attachKeyToArray(getWeek(currentWeek), 'days')], 'weeks');
    var dayWeeks = attachKeyToArray([attachKeyToArray([selectedDate], 'days')], 'weeks');

    switch (viewType) {
      case 'month':
        return {
          weeks: monthWeeks
        };

      case 'week':
        return {
          weeks: weekWeeks
        };

      case 'day':
      default:
        return {
          weeks: dayWeeks
        };
    }
  }, [currentDate, getCurrentWeek, getMonth, getWeek]);
  var setNext = react.useMemo(function () {
    switch (viewType) {
      case CalendarViewType.Month:
        return dateFns.addMonths;

      case CalendarViewType.Week:
        return dateFns.addWeeks;

      case CalendarViewType.Day:
        return dateFns.addDays;
    }
  }, [viewType]);
  var setPrev = react.useMemo(function () {
    switch (viewType) {
      case CalendarViewType.Month:
        return dateFns.subMonths;

      case CalendarViewType.Week:
        return dateFns.subWeeks;

      case CalendarViewType.Day:
        return dateFns.subDays;
    }
  }, [viewType]);
  return {
    calendar: _objectSpread(_objectSpread({}, calendar), {}, {
      viewType: viewType
    }),
    headers: getHeaders(viewType),
    body: getBody(viewType),
    navigation: {
      toNext: function toNext() {
        return setCurrentDate(function (date) {
          return setNext(date, 1);
        });
      },
      toPrev: function toPrev() {
        return setCurrentDate(function (date) {
          return setPrev(date, 1);
        });
      },
      setToday: function setToday() {
        return setCurrentDate(new Date());
      },
      setDate: function setDate(date) {
        return setCurrentDate(date);
      }
    },
    view: {
      type: viewType,
      setViewType: setViewType,
      setWeekStartsOn: setWeekStartsOn,
      isMonthView: viewType === CalendarViewType.Month,
      isWeekView: viewType === CalendarViewType.Week,
      isDayView: viewType === CalendarViewType.Day,
      showMonthView: function showMonthView() {
        return setViewType(CalendarViewType.Month);
      },
      showWeekView: function showWeekView() {
        return setViewType(CalendarViewType.Week);
      },
      showDayView: function showDayView() {
        return setViewType(CalendarViewType.Day);
      }
    }
  };
}

exports.CalendarViewType = CalendarViewType;
exports.default = useCalendar;
//# sourceMappingURL=index.ts.map
